@@ -35,7 +35,6 @@ import logging
 import time
 import datetime
 import sys
-import warnings
 
 import numpy as np
 
@@ -532,6 +531,8 @@ class Trials(object):
             loss_v = fmap(bandit.loss_variance)
             true_loss = fmap(bandit.true_loss)
         loss3 = zip(loss, loss_v, true_loss)
+        if not loss3:
+            raise ValueError('Empty loss vector')
         loss3.sort()
         loss3 = np.asarray(loss3)
         if np.all(loss3[:, 1] == 0):
@@ -603,7 +604,6 @@ class Ctrl(object):
         self.current_trial = current_trial
 
     def checkpoint(self, r=None):
-        # warnings.warn('Ctrl.checkpoint base implementation is no-op')
         assert self.current_trial in self.trials._trials
         if r is not None:
             self.current_trial['result'] = r
