@@ -135,7 +135,10 @@ __license__ = "3-clause BSD License"
 __contact__ = "github.com/jaberg/hyperopt"
 
 import copy
-import cPickle
+try:
+    import dill as cPickle
+except ImportError:
+    import cPickle
 import datetime
 import hashlib
 import logging
@@ -318,7 +321,7 @@ def coarse_utcnow():
     """
     # MongoDB stores only to the nearest millisecond
     # This is mentioned in a footnote here:
-    # http://api.mongodb.org/python/1.9%2B/api/bson/son.html#dt
+    # http://api.mongodb.org/python/current/api/bson/son.html#dt
     """
     now = datetime.datetime.utcnow()
     microsec = (now.microsecond//10**3)*(10**3)
@@ -1093,7 +1096,11 @@ class MongoWorker(object):
                             kwargs=bandit_kwargs).evaluate
                 elif cmd_protocol == 'domain_attachment':
                     blob = ctrl.trials.attachments[cmd[1]]
-                    domain = cPickle.loads(blob)
+                    try:
+                        domain = cPickle.loads(blob)
+                    except BaseException, e:
+                        logger.info('Error while unpickling. Try installing dill via "pip install dill" for enhanced pickling support.')
+                        raise
                     worker_fn = domain.evaluate
                 else:
                     raise ValueError('Unrecognized cmd protocol', cmd_protocol)
@@ -1235,7 +1242,8 @@ def main_worker_helper(options, args):
                 sub_argv = [sys.argv[0],
                         '--poll-interval=%s' % options.poll_interval,
                         '--max-jobs=1',
-                        '--mongo=%s' % options.mongo]
+                        '--mongo=%s' % options.mongo,
+                        '--reserve-timeout=%s' % options.reserve_timeout]
                 if options.workdir is not None:
                     sub_argv.append('--workdir=%s' % options.workdir)
                 if options.exp_key is not None:
@@ -1438,7 +1446,11 @@ def main_search_helper(options, args, input=input, cmd_type=None):
             if bandit_NAK != atup:
                 raise BanditSwapError((bandit_NAK, atup))
         else:
-            blob = cPickle.dumps(bandit_NAK)
+            try:
+                blob = cPickle.dumps(bandit_NAK)
+            except BaseException, e:
+                print >> sys.stdout, "Error pickling. Try installing dill via 'pip install dill'."
+                raise e
             trials.attachments[aname] = blob
         worker_cmd = ('driver_attachment', aname)
     else:
