@@ -332,6 +332,7 @@ class Apply(object):
             binding[param_i] = arg_i
 
         if extra_args_ok:
+            # XXX: THIS IS NOT BEING TESTED AND IS OBVIOUSLY BROKEN
             binding[args_param].extend(args[code.co_argcount:])
 
         # -- bind keyword arguments
@@ -480,7 +481,7 @@ class Apply(object):
 
     def __len__(self):
         if self.o_len is None:
-            return object.__len__(self)
+            raise TypeError('len of pyll.Apply either undefined or unknown')
         return self.o_len
 
     def __call__(self, *args, **kwargs):
@@ -1105,6 +1106,16 @@ def switch(pos, *args):
     return args[pos]
 
 
+def _kwswitch(kw, **kwargs):
+    """conditional evaluation according to string value"""
+    # Get the index of the string in kwargs to use switch
+    keys, values = zip(*sorted(kwargs.iteritems()))
+    match_idx = scope.call_method_pure(keys, 'index', kw)
+    return scope.switch(match_idx, *values)
+
+scope.kwswitch = _kwswitch
+
+
 @scope.define_pure
 def Raise(etype, *args, **kwargs):
     raise etype(*args, **kwargs)
