@@ -25,3 +25,6 @@ def test_landing_screen():
 
     print best
     # -> {'a': 1, 'c2': 0.01420615366247227}
+
+    print hyperopt.space_eval(space, best)
+    # -> ('case 2', 0.01420615366247227}
