@@ -19,6 +19,7 @@ from .base import BanditAlgo
 from .base import STATUS_OK
 from .base import miscs_to_idxs_vals
 from .base import miscs_update_idxs_vals
+from .base import Trials
 import rand
 
 logger = logging.getLogger(__name__)
@@ -49,14 +50,13 @@ def adaptive_parzen_sampler(name):
 # -- Categorical
 
 @scope.define
-def categorical_lpdf(sample, p):
+def categorical_lpdf(sample, p, upper):
     """
-    Return a random integer from 0 .. N-1 inclusive according to the
-    probabilities p[0] .. P[N-1].
-
-    This is formally equivalent to np.where(multinomial(n=1, p))
     """
-    return np.log(np.asarray(p)[sample])
+    if sample.size:
+        return np.log(np.asarray(p)[sample])
+    else:
+        return np.asarray([])
 
 
 # -- Bounded Gaussian Mixture Model (BGMM)
@@ -573,16 +573,41 @@ def ap_qlognormal_sampler(obs, prior_weight, mu, sigma, q, size=(), rng=None):
 # -- Categorical
 
 @adaptive_parzen_sampler('randint')
-def ap_categorical_sampler(obs, prior_weight, upper, size=(), rng=None,
-        LF=DEFAULT_LF):
+def ap_categorical_sampler(obs, prior_weight, upper,
+        size=(), rng=None, LF=DEFAULT_LF):
     weights = scope.linear_forgetting_weights(scope.len(obs), LF=LF)
     counts = scope.bincount(obs, minlength=upper, weights=weights)
     # -- add in some prior pseudocounts
     pseudocounts = counts + prior_weight
     return scope.categorical(pseudocounts / scope.sum(pseudocounts),
-            size=size, rng=rng)
+            upper=upper, size=size, rng=rng)
 
 
+# @adaptive_parzen_sampler('categorical')
+# def ap_categorical_sampler(obs, prior_weight, p, upper, size=(), rng=None,
+#                            LF=DEFAULT_LF):
+#     return scope.categorical(p, upper, size=size, rng
+#                              =rng)
+
+@scope.define
+def tpe_cat_pseudocounts(counts, upper, prior_weight, p, size):
+    #print counts
+    if size == 0 or np.prod(size) == 0:
+        return []
+    if p.ndim == 2:
+        assert np.all(p == p[0])
+        p = p[0]
+    pseudocounts = counts + upper * (prior_weight * p)
+    return pseudocounts / np.sum(pseudocounts)
+
+@adaptive_parzen_sampler('categorical')
+def ap_categorical_sampler(obs, prior_weight, p, upper=None,
+        size=(), rng=None, LF=DEFAULT_LF):
+    weights = scope.linear_forgetting_weights(scope.len(obs), LF=LF)
+    counts = scope.bincount(obs, minlength=upper, weights=weights)
+    pseudocounts = scope.tpe_cat_pseudocounts(counts, upper, prior_weight, p, size)
+    return scope.categorical(pseudocounts, upper=upper, size=size, rng=rng)
+
 #
 # Posterior clone performs symbolic inference on the pyll graph of priors.
 #
@@ -788,9 +813,23 @@ def suggest(new_ids, domain, trials,
         ):
 
     if len(new_ids) > 1:
-        # write a loop to draw new points sequentially
-        # TODO: insert constant liar for tentative suggestions
-        raise NotImplementedError('generates one at a time')
+        # -- greedy loop rolling forward
+        trials_copy = Trials()
+        trials_copy._dynamic_trials = trials.trials
+        trials_copy.refresh()
+        rval = []
+        for new_id in new_ids:
+            new_trials1 = suggest([new_id], domain, trials_copy,
+                seed=seed,
+                prior_weight=prior_weight,
+                n_startup_jobs=n_startup_jobs,
+                n_EI_candidates=n_EI_candidates,
+                gamma=gamma,
+                linear_forgetting=linear_forgetting)
+            trials_copy.insert_trial_docs(new_trials1)
+            trials_copy.refresh()
+            rval.extend(new_trials1)
+        return rval
     else:
         new_id, = new_ids
 
@@ -859,7 +898,8 @@ def suggest(new_ids, domain, trials,
     memo[observed_loss['idxs']] = tids
     memo[observed_loss['vals']] = losses
 
-    idxs, vals = pyll.rec_eval([opt_idxs, opt_vals], memo=memo)
+    idxs, vals = pyll.rec_eval([opt_idxs, opt_vals], memo=memo,
+            print_node_on_error=False)
 
     # -- retrieve the best of the samples and form the return tuple
     # the build_posterior makes all specs the same
